@@ -3,7 +3,9 @@
 // |' | '|   (c) 2020 - present, Vladimir Zvezda
 //   / \
 
+pub mod any_of_t;
+pub mod async_drop_t;
 pub mod channel_t;
+pub mod future_utils;
 pub mod oneshot_t;
 pub mod spawn_t;
-pub mod future_utils;
